@@ -38,6 +38,9 @@ static const struct {
 	{"'\\s*(and|or|xor|&&|\\|\\|)\\s*('|[0-9]|`?[a-z\\._-]+`?\\s*=|[a-z]+\\s*\\()"},
 	{"'\\s*like\\s*['a-z]"},
 	{"'\\s*(not\\s+)?in\\s*\\(\\s*['0-9]"},
+	{"select\\s+`?(\\*|[a-z0-9_,]*)`?\\s+from\\s+`?[a-z0-9_\\.]*"},
+	{"update\\s+`?[a-z0-9_\\.]*`?\\s+set\\s+"},
+	{"delete\\s+from\\s+`?[a-z0-9_\\.]*`?"},
 	{NULL}
 };
 
